## Install the lateset version of the BAS package from Github
# library(devtools)
# install_github("merliseclyde/BAS")

library(BAS);
library(AUC);

#####################################################
########      Read the GUSTO-I data          ########
#####################################################
#### The original data is in the "gusto" folder.
#### The .rda file of the dataset is in the "data" folder.

## Use the "foreign" library to read the .sav file
library(foreign);
gusto = read.spss('gusto/GustoW.sav', to.data.frame=T);

## See Steyerberg [2009] book: Clinical Prediction Models, Ch22 for details, page 454
dim(gusto) # n = 2188, p = 26

#### clean data to be the same as Held et al [2015]
## y: DAY30
## x1: SEX
## x2: AGE
## x3: KILLIP
## x4: DIA
## x5: HYP
## x6: HRT
## x7: ANT
## x8: PMI
## x9: HEI
## x10: WEI
## x11: HTN
## x12: SMK
## x13: LIP
## x14: PAN
## x15: FAM
## x16: STE
## x17: TTR

attach(gusto);
temp = cbind(DAY30, SEX, AGE, KILLIP, DIA, HYP, HRT, ANT, PMI, HEI, WEI, HTN, SMK, LIP, PAN, FAM, STE, TTR);
detach(gusto);
colnames(temp) = c('y', paste('x', 1:17, sep = ''));
gusto = temp;
save(gusto, file = 'data/gusto.rda');

#####################################################
########  Bootstrap Indices: for prediciton  ########
#####################################################
## We sample 1000 independnet bootstrap samples,
## and record the indices in the file "bootstrap_index.rda".

n = dim(gusto)[1];

set.seed(1);
bi = matrix(NA, nrow = 1000, ncol = n);
for(i in 1:1000){
  bi[i, ] = sample(1:n, n, replace = TRUE);
}

nti = rep(NA, 1000);
for(i in 1:1000){
  nti[i] = n - length(unique(bi[i, ]));
}
save(bi, nti, file = 'data/bootstrap_index.rda');


##########################################################################
#####          Prediction: using bootstrap CV                       ######
##########################################################################
# rm(list = ls());
# load(file = 'data/gusto.rda');
# load(file = 'data/bootstrap_index.rda');
n = as.numeric(dim(gusto)[1]); p = dim(gusto)[2] - 1;
gusto = as.data.frame(gusto);


pred = array(NA, dim = c(18, 5, 2, 1000));
## First dimension: priors on beta. 
####  c('CH(a=1/2,b=n)', 'CH(a=1,b=n)', 'CH(a=1/2,b=n/2)', 'CH(a=1,b=n/2)', 'Beta-prime', 'ZS adpated', 'Robust', 'Intrinsic', 'Hyper-g/n', 'Benchmark', 'DBF g=n', 'TBF g=n', 'Jeffreys', 'Hyper-g', 'Uniform', 'local EB', 'AIC', 'BIC');
## Second dimension: measurements.
####  c('AUC', 'CS', 'LS', 'Brier', 'elapsed_time');
## Third dimension: prior on the model space.
####  c('uniform', 'adjusted');
## Fourth dimension: bootstrap datasets

## priors on the model space
modelpriors = list(uniform(), beta.binomial(1, 1)); 

## priors on beta
betapriors = list(
	CCH(a = 1 / 2, b = n, s = 0), 
	CCH(a = 1, b = n, s = 0), 
	CCH(a = 1 / 2, b = n / 2, s = 0), 
	CCH(a = 1, b = n / 2, s = 0), 
	beta.prime(n = n), 
	CCH(a = 1, b = 2, s = n + 3),
	robust(n = n), 
	intrinsic(n = n), 
	hyper.g.n(alpha = 3, n = n), 
	CCH(a = 0.02, b = 0.02 * max(n, p^2), s = 0),
	g.prior(n), 
	testBF.prior(n),
	Jeffreys(), 
	CCH(a = 1, b = 2, s = 0), 
	CCH(a = 2, b = 2, s = 0), 
	EB.local(), 
	aic.prior(), 
	bic.prior(n)
);

########################################################################
## Folder 1 - 1000
for(r in 1:1000){

  # r = 1;
  load('r.rda');
  bootstrap.index = bi[r, ];
  test.index = (1:n)[-unique(bootstrap.index)];

  for(l in 1:2){ ## different model priors
    for(k in 1:18){ ## different betaprior 
      # k = 1; l = 1;
      start.time = proc.time();
      results = bas.glm(y ~ ., data = gusto[bootstrap.index, ], betaprior = betapriors[[k]], n.models = 2^p, method = "BAS", family = binomial(link = 'logit'), modelprior = modelpriors[[l]], initprobs = "eplogp", laplace = FALSE);

      results.test = predict.basglm(results, newdata = gusto[test.index, ], type = "link");
      test.mu = 1 / (1 + exp(-results.test$Ybma));
      ## AUC
      pred[k, 1, l, r] = auc(roc(test.mu, factor(gusto[test.index, 1])));
      ## CS
      pred[k, 2, l, r] = glm(gusto[test.index, 1] ~ log(test.mu / (1 - test.mu)), family = binomial(link = logit))$coef[2];
      ## LS
      pred[k, 3, l, r] = -mean(gusto[test.index, 1] * log(test.mu) + (1 - gusto[test.index, 1]) * log(1 - test.mu));
      ## Brier score
      pred[k, 4, l, r] = mean((gusto[test.index, 1] - test.mu)^2);
    
      # save(pred, file = 'GUSTO_predict.rda');

      rm(results);
      rm(results.test);
    }
  }
}            
