library(mvtnorm);
library(BAS);

rm(list = ls());
#############################################################################################################
############################          New Simulation for Logistic regression      ###########################
#############################################################################################################

n = 500; 
p.full = c(rep(20, 4), 100);

## Covariance matrix for correlated X
Sigma = matrix(NA, nr = 100, nc = 100);
for(i in 1:100){
	for(j in 1:100){
		Sigma[i,j] = 0.75^(abs(i-j));
	}
}

## Note: the betatrue here include the intercpet.
Betatrue = matrix(0, ncol = 20 + 1, nrow = 4);
colnames(Betatrue) = c('alpha', paste('beta', 1:20, sep = ''));
Betatrue[, 1] = -0.5;
Betatrue.tmp = c(2, -1, -1, 0.5, -0.5);
Betatrue[2:4, 1:5 + 1] = rep(Betatrue.tmp, rep(3, length(Betatrue.tmp)));
Betatrue[3:4, 11:15 + 1] = rep(Betatrue.tmp, rep(2, length(Betatrue.tmp)));
Betatrue[4, c(6:10, 16:20) + 1] = rep(Betatrue.tmp * 0.5, 2);
## Note: when p = 100, use Betatrue[2, ].
Betatrue = rbind(Betatrue, Betatrue[2, ]);

#############################################################################################################
################      For each i = 1, ... 5, each j = 1 or 2, simulate 100 realizations      ################
#############################################################################################################

X.full = NULL;
y.full = NULL;
for(i in 1:5){ ## different betatrue
  for(j in 1:2){ ## 1: indepdent X; 2: correlated X, rho = 0.75	
    for(r in 1:100){ ## for each scenario, generate 100 datasets
  	  
      #r = 1; i = 1; j = 1;
      X = (j == 1) * rmvnorm(n, sigma = diag(1, 100)) + (j == 2) * rmvnorm(n, sigma = Sigma); 
      X = scale(X);
      theta = Betatrue[i, 1] + X[ , 1:20] %*% Betatrue[i, -1];
      y = rbinom(n, 1, 1 / (1 + exp(-theta)));
    
      X.full[[(2 * (i - 1) + j - 1) * 100 + r]] = X; 
      y.full[[(2 * (i - 1) + j - 1) * 100 + r]] = y;
    }
  }
}

save(X.full, y.full, Betatrue, n, p.full, file = '073016_logistic_data.rda');
## I should have set a seed, but I fotgot to do so ... 
## The file "073016_logistic_data.rda" is over 380MB, so I didn't included it in the zip file. It's available upon request.


#############################################################################################################
#######################              Use BAS package to fit the data               ##########################
#############################################################################################################

## priors on the model space
modelpriors = list(uniform(), beta.binomial(1, 1)); 

## priors on beta
betapriors = list(
	CCH(a = 1 / 2, b = n, s = 0), 
	CCH(a = 1, b = n, s = 0), 
	CCH(a = 1 / 2, b = n / 2, s = 0), 
	CCH(a = 1, b = n / 2, s = 0), 
	beta.prime(n = n), 
	CCH(a = 1, b = 2, s = n + 3),
	robust(n = n), 
	intrinsic(n = n), 
	hyper.g.n(alpha = 3, n = n), 
	CCH(a = 0.02, b = 0.02 * max(n, p^2), s = 0),
	g.prior(n), 
	testBF.prior(n),
	Jeffreys(), 
	CCH(a = 1, b = 2, s = 0), 
	CCH(a = 2, b = 2, s = 0), 
	EB.local(), 
	aic.prior(), 
	bic.prior(n)
);

#### We first create arrays to save results. For the following objects,
#### dim1 is betaprior, dim2 is beta (may include intercept), dim3 is modelprior, dim4 is number of datasets
## Results for the scenarios where p = 20
Top20 = Incprob20 = array(NA, dim = c(18, 20, 2, 800)); 
BMA20 = array(NA, dim = c(18, 21, 2, 800));
## Results for the scenarios where p = 100
Top100 = Incprob100 = array(NA, dim = c(18, 100, 2, 200));
BMA100 = array(NA, dim = c(18, 101, 2, 200));

#### Summary of results:
#### dim1 is betaprior, dim2 is measurements, dim3 is modelprior, dim4 is number of datasets
#### The five measurements are: c('sture', 'topsize', 'l2est', 'l1est', 'runtime')
Analysis = array(NA, dim = c(18, 5, 2, 1000));

#############################################################################################################
for(r in 1:100){ ## 100 datasets for each scenario
  for(i in 1:5){ ## different betatrue
    for(j in 1:2){ ## 1: indepdent X; 2: correlated X, rho = 0.75	

  	  ind = (2 * (i - 1) + j - 1) * 100 + r;
  	  p = p.full[i];
  	  betatrue = Betatrue[i, ];
      X = X.full[[ind]][, 1:p];
      y = y.full[[ind]];
      Xy = data.frame(X, y);
      
      for(l in 1:2){ ## different model priors
        for(k in 1:18){ ## different betaprior 
      
          start.time = proc.time();
          if(p == 20)
            results = bas.glm(y ~ ., data = Xy, betaprior = betapriors[[k]], method = 'BAS', family = binomial(link = 'logit'), modelprior = modelpriors[[l]], n.models = 2^20, initprobs = 'eplogp', laplace = FALSE);
          if(p == 100)
            results = bas.glm(y ~ ., data = Xy, betaprior = betapriors[[k]], method = 'MCMC', family = binomial(link = 'logit'), modelprior = modelpriors[[l]], n.models = 2^17, MCMC.iterations = 2^17, initprobs = 'eplogp', laplace = FALSE);
      
          Analysis[k, 5, l, ind] = (proc.time() - start.time)[3];

          ## model selection
          top = summary(results)[1, 2:(p + 1)];  ## the MAP model
          Analysis[k, 2, l, ind] = sum(top); ## size of the MAP model
          yes.top = sum(top[yes.betatrue]);
          no.top = sum(top[no.betatrue]);      
          yes.betatrue = which(betatrue[-1] != 0);
          no.betatrue = which(betatrue[-1] == 0);
          if(yes.top == length(yes.betatrue) & no.top == 0){ ## whether the MAP model is the true model
            Analysis[k, 1, l, ind] = 1;	
          } else{
            Analysis[k, 1, l, ind] = 0;	
          }
    
          ## BMA parameter estimation    
          tmp = coef(results);
          bma = tmp[[1]];
          if(p == 20){
            Analysis[k, 3, l, ind] = sum((betatrue - bma)^2);
            Analysis[k, 4, l, ind] = sum(abs(betatrue - bma));
          }
          if(p == 100){
            Analysis[k, 3, l, ind] = sum((c(betatrue, rep(0, 80)) - bma)^2);
            Analysis[k, 4, l, ind] = sum(abs(c(betatrue, rep(0, 80)) - bma));
          }
    
          ## posterior marginal includsion probability
          incprob = tmp[[3]][-1];
      
          ## save results
          if(p == 20){
  		    Top20[k, , l, ind] = top;
  		    Incprob20[, , , ind] = incprob;
  		    BMA20[, , , ind] = bma;
  	      }
          if(p == 100){
  	        Top100[, , , ind - 800] = top;
  	        Incprob100[, , , ind - 800] = incprob;
  	        BMA100[, , , ind - 800] = bma;
  	      }
  	      
  	      rm(results);
  	    }  
      }

      save(Top20, Top100, Incprob20, Incprob100, BMA20, BMA100, Analysis, file = 'results/simulation_logistic_results_selection_estimation.rda');
    }  
  }
}



